@@ -1,19 +1,22 @@
 package Test::DBIx::Class::SchemaManager::Trait::Testmysqld;
-	
+
 use Moose::Role;
 use MooseX::Attribute::ENV;
 use Test::mysqld;
 use Test::More ();
 use Path::Class qw(dir);
 use Test::DBIx::Class::Types qw(ReplicantsConnectInfo);
+use Socket;
+use File::Temp qw(tempdir);
+use File::Path qw(make_path);
 
 requires 'setup', 'cleanup';
 
 ## has '+force_drop_table' => (is=>'rw',default=>1);
 
 has [qw/base_dir mysql_install_db mysqld/] => (
-    is=>'ro', 
-    traits=>['ENV'], 
+    is=>'ro',
+    traits=>['ENV'],
 );
 
 has test_db_manager => (
@@ -38,15 +41,15 @@ sub _build_test_db_manager {
 }
 
 has default_cnf => (
-    is=>'ro', 
-    init_arg=>undef, 
-    isa=>'HashRef', 
-    auto_deref=>1, 
+    is=>'ro',
+    init_arg=>undef,
+    isa=>'HashRef',
+    auto_deref=>1,
     lazy_build=>1,
 );
 
 sub _build_default_cnf {
-    my $port = first_unused_port();
+    my $port = $_[0]->find_next_unused_port();
     return {
         'server-id'=>1,
         'log-bin'=>'mysql-bin',
@@ -55,9 +58,14 @@ sub _build_default_cnf {
     };
 }
 
+has port_to_try_first => (
+    is=>'rw',
+    default=> sub { 8000 + int(rand(2000)) },
+);
+
 has my_cnf => (
-    is=>'ro', 
-    isa=>'HashRef', 
+    is=>'ro',
+    isa=>'HashRef',
     auto_deref=>1,
 );
 
@@ -97,29 +105,29 @@ has balancer_args => (
         return {
             auto_validate_every=>10,
             master_read_weight => 1,
-        },	
+        },
     },
 );
 
 has default_replicant_cnf => (
-    is=>'ro', 
-    init_arg=>undef, 
-    isa=>'HashRef', 
-    auto_deref=>1, 
+    is=>'ro',
+    init_arg=>undef,
+    isa=>'HashRef',
+    auto_deref=>1,
     required=>1,
     default=> sub { +{} },
 );
 
 has my_replicant_cnf => (
-    is=>'ro', 
-    isa=>'HashRef', 
+    is=>'ro',
+    isa=>'HashRef',
     auto_deref=>1,
 );
 
 sub prepare_replicant_config {
     my ($self, $replicant, @replicants,%extra) = @_;
     my %my_cnf_extra = $extra{my_cnf} ? delete $extra{my_cnf} : ();
-    my $port = first_unused_port();
+    my $port = find_next_unused_port();
     my %config = (
         my_cnf => {
             'port'=>$port,
@@ -132,12 +140,12 @@ sub prepare_replicant_config {
     );
 
     my $replicant_name =$replicant->{name};
-    my $base_dir = $self->test_db_manager->base_dir . "/$replicant_name";
+    my $base_dir = $self->test_db_manager->base_dir . "/$port" .'_'. "$replicant_name";
+
+    $config{base_dir} = $base_dir;
+    $config{mysql_install_db} = $self->mysql_install_db if $self->mysql_install_db;
+    $config{mysqld} = $self->mysqld if $self->mysqld;
 
-    $config{base_dir} = $base_dir;	
-    $config{mysql_install_db} = $self->mysql_install_db if $self->mysql_install_db;	
-    $config{mysqld} = $self->mysqld if $self->mysqld;	
-    
     return %config;
 }
 
@@ -159,13 +167,13 @@ around 'prepare_schema_class' => sub {
 
 around 'setup' => sub {
     my ($setup, $self, @args) = @_;
- 
+
     return $self->$setup(@args) unless $self->has_replicants;
 
     ## Do we need to invent replicants?
     my @replicants = ();
     my @deployed_replicants = ();
-    foreach	my $replicant ($self->replicants) { 
+    foreach	my $replicant ($self->replicants) {
         if($replicant->{dsn}) {
             push @replicants, $replicant;
         } else {
@@ -185,10 +193,10 @@ around 'setup' => sub {
 
             Test::More::diag("DBI->connect('DBI:mysql:test;mysql_socket=$replicant_base_dir/tmp/mysql.sock','root','')");
             push @deployed_replicants, $deployed;
-            push @replicants, 
+            push @replicants,
               ["DBI:mysql:test;mysql_socket=$replicant_base_dir/tmp/mysql.sock",'root',''];
 
-        }	
+        }
     }
 
 
@@ -196,12 +204,13 @@ around 'setup' => sub {
     $self->replicants(\@replicants);
     $self->schema->storage->connect_replicants($self->replicants);
     $self->schema->storage->ensure_connected;
+    my $port =  $self->default_cnf->{port};  ## TODO I doubt this is correct.....
 
     foreach my $storage ($self->schema->storage->pool->all_replicant_storages) {
         ## TODO, need to change this to dbh_do
         my $dbh = $storage->_get_dbh;
         $dbh->do("STOP SLAVE") || die $dbh->errst;
-        $dbh->do("CHANGE MASTER TO  master_host='127.0.0.1',  master_port=8001,  master_user='root',  master_password=''")
+        $dbh->do("CHANGE MASTER TO  master_host='127.0.0.1',  master_port=$port,  master_user='root',  master_password=''")
             || die $dbh->errstr;
         $dbh->do("START SLAVE")
             || die $dbh->errstr;
@@ -215,17 +224,19 @@ sub prepare_config {
     my %my_cnf_extra = $extra{my_cnf} ? delete $extra{my_cnf} : ();
     my %config = (
         my_cnf => {
-            $self->default_cnf, 
-            $self->my_cnf, 
+            $self->default_cnf,
+            $self->my_cnf,
             %my_cnf_extra,
         },
         %extra,
     );
+    my $port = $self->default_cnf->{port};
+    my $cleanup = ($ENV{TEST_MYSQLD_PRESERVE} || $self->keep_db) ? undef : 1;
+    $config{base_dir} = $self->base_dir ? $self->base_dir ."/$port" : tempdir(CLEANUP => $cleanup) . "/$port";
+    make_path($config{base_dir}) unless -e $config{base_dir};
+    $config{mysql_install_db} = $self->mysql_install_db if $self->mysql_install_db;
+    $config{mysqld} = $self->mysqld if $self->mysqld;
 
-    $config{base_dir} = $self->base_dir if $self->base_dir;	
-    $config{mysql_install_db} = $self->mysql_install_db if $self->mysql_install_db;	
-    $config{mysqld} = $self->mysqld if $self->mysqld;	
- 
     return %config;
 }
 
@@ -261,13 +272,12 @@ after 'cleanup' => sub {
     }
 };
 
-use Socket;
 sub is_port_open {
     my ($port) = @_;
     my ($host, $iaddr, $paddr, $proto);
 
     $host  =  '127.0.0.1';
-    $iaddr   = inet_aton($host)               
+    $iaddr   = inet_aton($host)
         or die "no host: $host";
     $paddr   = sockaddr_in($port, $iaddr);
 
@@ -279,12 +289,12 @@ sub is_port_open {
             or die "error closing test socket: $!";
         return 1;
     }
-    return 0; 
+    return 0;
 }
 
-our $next_port = 8000 + int(rand(2000));
-sub first_unused_port {
-    ++$next_port;
+our $next_port;
+sub find_next_unused_port {
+    $next_port ||= $_[0]->port_to_try_first;
     my $port = $next_port;
     while (is_port_open($port)) {
         $port++;
@@ -292,6 +302,7 @@ sub first_unused_port {
             die "no ports available\n";
         }
     }
+    ++$next_port;
     return $port;
 }
 
@@ -316,8 +327,8 @@ Please review L<Test::mysqld> for help if you get stuck.
 =head1 CONFIGURATION
 
 This trait supports all the existing features but adds some additional options
-you can put into your inlined of configuration files.  These following 
-additional configuration options basically map to the options supported by 
+you can put into your inlined of configuration files.  These following
+additional configuration options basically map to the options supported by
 L<Test::mysqld> and the docs are adapted shamelessly from that module.
 
 For the most part, if you have mysql installed in a normal, findable manner
@@ -355,6 +366,18 @@ you have some specific needs you can leave this empty, since we set the few
 things most needed to get a server running.  You will need to review the
 documentation on the Mysql website for options related to this.
 
+=head2 port_to_try_first
+
+This is the port that will be used when starting mysqld. We check that this
+port is available for use before starting mysqld. If it is not available we
+increment by 1 and try again. We use the first free port found.
+
+By default this is a random port between 8000 and 10000. The randomness is
+an attempt to avoid race condition issues when running tests in parallel,
+between checking the availability of a port and actually starting the server.
+Spreading the "first port" numbers used greatly reduces the chance of these
+issues occuring.
+
 =head2 mysql_install_db or mysqld
 
 If your mysqld is not in the $PATH you might need to specify the location to
@@ -369,14 +392,14 @@ in the future)
 
 =head1 NOTES
 
-The following are notes regarding the way this trait alters or extends the 
+The following are notes regarding the way this trait alters or extends the
 core functionality as described in the basic documentation.
 
 =head2 force_drop_table
 
 Since it is always safe to use the 'force_drop_table' option with mysql, we set
 the default to true.  We recommend you leave it this way, particularly if you
-want to 'roundtrip' the same test database. 
+want to 'roundtrip' the same test database.
 
 =head2 keep_db
 
@@ -412,7 +435,7 @@ You can now log into the test generated database instance with:
 
 	mysql --socket=/tmp/WT0P0VutAe/tmp/mysql.sock -u root test
 
-You may need to specify the full path to 'mysql' if it's not in your search 
+You may need to specify the full path to 'mysql' if it's not in your search
 $PATH.
 
 When you are finished you can then kill the process.  In this case our reported
@@ -424,7 +447,7 @@ And then you might wish to 'tidy' up temp
 
 	rm -rf /tmp/WT0P0VutAe
 
-All the above assume you are on a unix or unixlike system.  Would welcome 
+All the above assume you are on a unix or unixlike system.  Would welcome
 document patches for how to do all the above on windows.
 
 =head1 AUTHOR
